use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Database::Config;

use Clarity -self;

field db_type => 'Pg';
field db_name => undef;
field db_user => undef;
field db_pass => undef;

*set_db_type = *db_type;
*set_db_name = *db_name;
*set_db_user = *db_user;
*set_db_pass = *db_pass;

sub db_datasource {
	return $self->{db_datasource} = $_[0] if @_;
	return $self->{db_datasource} if exists $self->{db_datasource};

	my $type = $self->db_type // confess "undefined type";
	my $name = $self->db_name // '';
	return "dbi:$type:dbname=$name"
}

*set_db_datasource = *db_datasource;

field db_options => {
	PrintError => 0,
	RaiseError => 1,
	AutoCommit => 0,
	ShowErrorStatement => 1,
	pg_enable_utf8 => 1
};

package Xyzzy::Database::Request;

use Xyzzy::Request -self;

field db => sub { shift->cfg->conn };

package Xyzzy::Database;

use Xyzzy::Handler -self;

use Xyzzy::Database::Config;

use DBI;

field db_type => sub { shift->cfg->db_type };
field db_name => sub { shift->cfg->db_name };
field db_user => sub { shift->cfg->db_user };
field db_pass => sub { shift->cfg->db_pass };
field db_options => sub { shift->cfg->db_options };

*db_datasource = *Xyzzy::Database::Config::db_datasource;

field conn => sub {
	my $self = shift;
	return DBI->connect($self->db_datasource, $self->db_user, $self->db_pass, $self->db_options);
};

sub handle {
	my $req = new Xyzzy::Database::Request(ctx => shift, cfg => $self);
	if(my $db = $self->{conn}) {
		delete $self->{conn} unless $db->ping;
	}
	my $res = eval { super($req) };
	my $db = $self->{conn};
	if(defined $res) {
		if($db && $db->{Executed}) {
			$db->commit;
		}
	} else {
		if($db && $db->{Executed}) {
			local $@;
			eval { $db->rollback };
			warn $@ if $@;
		}
		die $@ if $@;
	}
	return $res;
}
